package com.broadlink.nj.flutter.plugin.blaccount_flutter_plugin;


import android.text.TextUtils;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import cn.com.broadlink.account.BLAccount;
import cn.com.broadlink.account.params.BLRegistParam;
import cn.com.broadlink.account.result.BLGetUserInfoResult;
import cn.com.broadlink.account.result.BLModifyUserIconResult;
import cn.com.broadlink.base.BLBaseResult;
import cn.com.broadlink.base.BLLoginResult;
import cn.com.broadlink.sdk.BLLet;


public class BLAccountTools {
    private final String LOGTAG = BLAccountTools.class.getSimpleName();
    private static BLAccountTools instance;

    private BLAccountTools() {

    }

    public static BLAccountTools getInstance() {
        if (instance == null) {
            synchronized (BLAccountTools.class) {
                if (instance == null) {
                    instance = new BLAccountTools();
                }
            }
        }
        return instance;
    }

    public boolean init(String initData){
        JsonParser jsonParser=new JsonParser();
        String companyId=null;
        String lid=null;
        try {
            JsonObject initDataJson=jsonParser.parse(initData).getAsJsonObject();
            JsonElement element=initDataJson.get("company_id");
            if(element!=null){
                companyId=element.getAsString();
            }
            element=initDataJson.get("lid");
            if(element!=null){
                lid=element.getAsString();
            }
            BLAccount.init(companyId, lid);
            BLAccount.addLoginListener(BLLet.Controller.getLoginListener());
            return true;
        }catch (Exception e){
            BLLogUtils.e(LOGTAG,"init BLSDK fail",e);
            JsonObject resultJson=new JsonObject();
            resultJson.addProperty("status", ResultStatus.InitSdkDataError);
            resultJson.addProperty("msg","init BLSDK data error");
        }
        return false;
    }

    public String login(String account, String password) {
        BLLoginResult blLoginResult = BLAccount.login(account, password);
        BLLogUtils.d(LOGTAG, "login:" + new Gson().toJson(blLoginResult));
        if (blLoginResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blLoginResult.getStatus());
            resultJson.addProperty("msg", blLoginResult.getMsg());

            JsonObject userInfoJson = new JsonObject();
            userInfoJson.addProperty("login_session", blLoginResult.getLoginsession());
            userInfoJson.addProperty("user_id", blLoginResult.getUserid());
            userInfoJson.addProperty("nickname", blLoginResult.getNickname());
            userInfoJson.addProperty("icon_path", blLoginResult.getIconpath());
            userInfoJson.addProperty("sex", blLoginResult.getSex());

            userInfoJson.addProperty("country_code", blLoginResult.getCountrycode());
            userInfoJson.addProperty("phone", blLoginResult.getPhone());
            userInfoJson.addProperty("email", blLoginResult.getEmail());
            userInfoJson.addProperty("birthday", blLoginResult.getBirthday());
            userInfoJson.addProperty("user_type", blLoginResult.getUsertype());

            resultJson.add("data", userInfoJson);
            return resultJson.toString();
        }
        return null;
    }

    public String logout(String account, String password) {
        BLLoginResult blLoginResult = BLAccount.login(account, password);
        BLLogUtils.d(LOGTAG, "logout:" + new Gson().toJson(blLoginResult));
        if (blLoginResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blLoginResult.getStatus());
            resultJson.addProperty("msg", blLoginResult.getMsg());

            if (blLoginResult.succeed()) {
                JsonObject userInfoJson = new JsonObject();
                userInfoJson.addProperty("login_session", blLoginResult.getLoginsession());
                userInfoJson.addProperty("user_id", blLoginResult.getUserid());
                userInfoJson.addProperty("nickname", blLoginResult.getNickname());
                userInfoJson.addProperty("icon_path", blLoginResult.getIconpath());
                userInfoJson.addProperty("sex", blLoginResult.getSex());

                userInfoJson.addProperty("country_code", blLoginResult.getCountrycode());
                userInfoJson.addProperty("phone", blLoginResult.getPhone());
                userInfoJson.addProperty("email", blLoginResult.getEmail());
                userInfoJson.addProperty("birthday", blLoginResult.getBirthday());
                userInfoJson.addProperty("user_type", blLoginResult.getUsertype());

                resultJson.add("data", userInfoJson);
            }
            return resultJson.toString();
        }
        return null;
    }

    public String localLogin(String loginSession, String userId) {
        BLLoginResult result = new BLLoginResult();
        result.setLoginsession(loginSession);
        result.setUserid(userId);
        BLLoginResult blLoginResult = BLAccount.localLogin(result);
        BLLogUtils.d(LOGTAG, "localLogin:" + new Gson().toJson(blLoginResult));
        if (blLoginResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blLoginResult.getStatus());
            resultJson.addProperty("msg", blLoginResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }


    public String sendRegVCode(String phone, String countryCode) {
        BLBaseResult blBaseResult = BLAccount.sendRegVCode(phone, countryCode);
        BLLogUtils.d(LOGTAG, "sendRegVCode:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String sendRegVCode(String email) {
        BLBaseResult blBaseResult = BLAccount.sendRegVCode(email);
        BLLogUtils.d(LOGTAG, "sendRegVCode:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String regist(String data) {
        String account = null;
        String password = null;
        String nickname = null;
        String country = null;
        String countryCode = null;
        String birthday = null;
        String iconPath = null;
        String iconFilePath = null;
        String code = null;

        JsonParser jsonParser = new JsonParser();
        try {
            JsonObject initDataJson = jsonParser.parse(data).getAsJsonObject();
            JsonElement element = initDataJson.get("account");
            if (element != null) {
                account = element.getAsString();
            }
            element = initDataJson.get("password");
            if (element != null) {
                password = element.getAsString();
            }
            element = initDataJson.get("nickname");
            if (element != null) {
                nickname = element.getAsString();
            }
            element = initDataJson.get("country");
            if (element != null) {
                country = element.getAsString();
            }
            element = initDataJson.get("country_code");
            if (element != null) {
                countryCode = element.getAsString();
            }
            element = initDataJson.get("birthday");
            if (element != null) {
                birthday = element.getAsString();
            }
            element = initDataJson.get("iconpath");
            if (element != null) {
                iconPath = element.getAsString();
            }
            element = initDataJson.get("icon_file_path");
            if (element != null) {
                iconPath = element.getAsString();
            }
            element = initDataJson.get("code");
            if (element != null) {
                code = element.getAsString();
            }
            BLRegistParam registParam = new BLRegistParam();
            registParam.setPhoneOrEmail(account);//必须
            registParam.setPassword(password);//必须
            registParam.setNickname(nickname);
            registParam.setCountry(country);
            registParam.setCountrycode(countryCode);
//            registParam.setSex(sex);
            registParam.setBirthday(birthday);
            registParam.setIconpath(iconPath);
            registParam.setCode(code);//必须

            File iconFile = new File(iconFilePath);
            BLLoginResult blLoginResult = BLAccount.regist(registParam, iconFile.exists() ? iconFile : null);
            BLLogUtils.d(LOGTAG, "regist:" + new Gson().toJson(blLoginResult));
            if (blLoginResult != null) {
                JsonObject resultJson = new JsonObject();
                resultJson.addProperty("status", blLoginResult.getStatus());
                resultJson.addProperty("msg", blLoginResult.getMsg());

                if (blLoginResult.succeed()) {
                    JsonObject userInfoJson = new JsonObject();
                    userInfoJson.addProperty("login_session", blLoginResult.getLoginsession());
                    userInfoJson.addProperty("user_id", blLoginResult.getUserid());
                    userInfoJson.addProperty("nickname", blLoginResult.getNickname());
                    userInfoJson.addProperty("icon_path", blLoginResult.getIconpath());
                    userInfoJson.addProperty("sex", blLoginResult.getSex());

                    userInfoJson.addProperty("country_code", blLoginResult.getCountrycode());
                    userInfoJson.addProperty("phone", blLoginResult.getPhone());
                    userInfoJson.addProperty("email", blLoginResult.getEmail());
                    userInfoJson.addProperty("birthday", blLoginResult.getBirthday());
                    userInfoJson.addProperty("user_type", blLoginResult.getUsertype());

                    resultJson.add("data", userInfoJson);
                }
                return resultJson.toString();
            }
            return null;
        } catch (Exception e) {
            BLLogUtils.e(LOGTAG, "regist fail", e);
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", ResultStatus.RegistDataError);
            resultJson.addProperty("msg", "regist data error");
            return resultJson.toString();
        }
    }


    public String sendResetPwdVCode(String account) {
        BLBaseResult blBaseResult = BLAccount.sendRetrieveVCode(account);
        BLLogUtils.d(LOGTAG, "sendResetPwdVCode:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String resetPwd(String account,String vcode, String newPassword) {
        BLLoginResult blLoginResult = BLAccount.retrievePassword(account,vcode,newPassword);
        BLLogUtils.d(LOGTAG, "resetPwd:" + new Gson().toJson(blLoginResult));
        if (blLoginResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blLoginResult.getStatus());
            resultJson.addProperty("msg", blLoginResult.getMsg());

            if (blLoginResult.succeed()) {
                JsonObject userInfoJson = new JsonObject();
                userInfoJson.addProperty("login_session", blLoginResult.getLoginsession());
                userInfoJson.addProperty("user_id", blLoginResult.getUserid());
                userInfoJson.addProperty("nickname", blLoginResult.getNickname());
                userInfoJson.addProperty("icon_path", blLoginResult.getIconpath());
                userInfoJson.addProperty("sex", blLoginResult.getSex());

                userInfoJson.addProperty("country_code", blLoginResult.getCountrycode());
                userInfoJson.addProperty("phone", blLoginResult.getPhone());
                userInfoJson.addProperty("email", blLoginResult.getEmail());
                userInfoJson.addProperty("birthday", blLoginResult.getBirthday());
                userInfoJson.addProperty("user_type", blLoginResult.getUsertype());

                resultJson.add("data", userInfoJson);
            }
            return resultJson.toString();
        }
        return null;
    }

    public String sendModifyEmailVCode(String email){
        BLBaseResult blBaseResult = BLAccount.sendModifyVCode(email);
        BLLogUtils.d(LOGTAG, "sendModifyEmailVCode:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String sendModifyPhoneVCode(String phone,String countryCode){
        BLBaseResult blBaseResult = BLAccount.sendModifyVCode(phone,countryCode);
        BLLogUtils.d(LOGTAG, "sendModifyPhoneVCode:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String modifyPhone(String phone,String countyCode,String vcode,String password){
        BLBaseResult blBaseResult = BLAccount.modifyPhone(phone,countyCode,vcode,password,null);
        BLLogUtils.d(LOGTAG, "modifyPhone:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String modifyEmail(String email,String vcode,String password){
        BLBaseResult blBaseResult = BLAccount.modifyEmail(email,vcode,password,null);
        BLLogUtils.d(LOGTAG, "modifyEmail:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }


    public String modifyPassword(String oldPassword,String newPassword){
        BLBaseResult blBaseResult = BLAccount.modifyPassword(oldPassword,newPassword);
        BLLogUtils.d(LOGTAG, "modifyPassword:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String modifyNickname(String nickname){
        BLBaseResult blBaseResult = BLAccount.modifyUserNickname(nickname);
        BLLogUtils.d(LOGTAG, "modifyNickname:" + new Gson().toJson(blBaseResult));
        if (blBaseResult != null) {
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", blBaseResult.getStatus());
            resultJson.addProperty("msg", blBaseResult.getMsg());
            return resultJson.toString();
        }
        return null;
    }

    public String modifyIcon(String filePath){
        File iconFile=new File(filePath);
        if(iconFile.exists()){
            BLModifyUserIconResult blModifyUserIconResult = BLAccount.modifyUserIcon(iconFile);
            BLLogUtils.d(LOGTAG, "modifyIcon:" + new Gson().toJson(blModifyUserIconResult));
            if (blModifyUserIconResult != null) {
                JsonObject resultJson = new JsonObject();
                resultJson.addProperty("status", blModifyUserIconResult.getStatus());
                resultJson.addProperty("msg", blModifyUserIconResult.getMsg());

                if (blModifyUserIconResult.succeed()) {
                    JsonObject userInfoJson = new JsonObject();
                    userInfoJson.addProperty("icon_path", blModifyUserIconResult.getIcon());

                    resultJson.add("data", userInfoJson);
                }
                return resultJson.toString();
            }  
            return null;
        }else{
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", ResultStatus.IconFileNotExist);
            resultJson.addProperty("msg", "icon file not exist");
            return resultJson.toString();
        }
    }

    public String queryUsersInfo(String data){
        JsonParser jsonParser = new JsonParser();
        try {
            JsonArray userIdArray = jsonParser.parse(data).getAsJsonArray();
                List<String> userIdList=new ArrayList<>();
                Iterator<JsonElement> iterator = userIdArray.iterator();
                while (iterator.hasNext()){
                    String userId=iterator.next().getAsString();
                    if(!TextUtils.isEmpty(userId)){
                        userIdList.add(userId);   
                    }
                }
                if(userIdList.size()>0){
                    BLGetUserInfoResult blGetUserInfoResult = BLAccount.getUserInfo(userIdList);
                    BLLogUtils.d(LOGTAG, "queryUserInfo:" + new Gson().toJson(blGetUserInfoResult));
                    if(blGetUserInfoResult!=null){
                        JsonObject resultJson = new JsonObject();
                        resultJson.addProperty("status", blGetUserInfoResult.getStatus());
                        resultJson.addProperty("msg", blGetUserInfoResult.getMsg());
                        if(blGetUserInfoResult.succeed()){
                            List<BLGetUserInfoResult.UserInfo> userInfoList = blGetUserInfoResult.getInfo();
                            if(userIdList!=null&&userIdList.size()>0){
                                JsonArray dataArray=new JsonArray();
                                for(BLGetUserInfoResult.UserInfo userInfo:userInfoList){
                                    JsonObject userInfoJson=new JsonObject();
                                    userInfoJson.addProperty("user_id",userInfo.getUserid());
                                    userInfoJson.addProperty("nickname",userInfo.getNickname());
                                    userInfoJson.addProperty("icon",userInfo.getIcon());
                                    dataArray.add(userInfoJson);
                                }
                                resultJson.add("data", dataArray);
                            }
                        }
                        return resultJson.toString();
                    }
                    return null;
                }else{
                    BLLogUtils.w(LOGTAG, "queryUserInfo fail，UserId isEmpty");
                    JsonObject resultJson = new JsonObject();
                    resultJson.addProperty("status", ResultStatus.UserIdEmpty);
                    resultJson.addProperty("msg", "UserId isEmpty");
                    return resultJson.toString();
                }
                
        }catch (Exception e){
            BLLogUtils.e(LOGTAG, "queryUserInfo fail", e);
            JsonObject resultJson = new JsonObject();
            resultJson.addProperty("status", ResultStatus.UserIdEmpty);
            resultJson.addProperty("msg", "UserId isEmpty");
            return resultJson.toString();
        }
    }
}
