package com.broadlink.nj.flutter.plugin.blaccount_flutter_plugin;

import androidx.annotation.NonNull;

import cn.com.broadlink.account.BLAccount;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * BlaccountFlutterPlugin
 */
public class BlaccountFlutterPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    private final String LOGTAG=BlaccountFlutterPlugin.class.getSimpleName();
    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "blaccount_flutter_plugin");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        BLLogUtils.d(LOGTAG,"onMethodCall:"+call.method);
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        }
        else if (call.method.equals(NativeInterface.INIT_SDK)) {
            String data = call.argument("data");
            boolean init_result = BLAccountTools.getInstance().init(data);
            result.success(init_result);
        }
        else if (call.method.equals(NativeInterface.LOGIN)) {
            String account = call.argument("account");
            String password = call.argument("password");
            String login_result = BLAccountTools.getInstance().login(account,password);
            result.success(login_result);
        }
        else if (call.method.equals(NativeInterface.LOCAL_LOGIN)) {
          String loginSession = call.argument("login_session");
          String userId = call.argument("user_id");
          String login_result = BLAccountTools.getInstance().localLogin(loginSession,userId);
          result.success(login_result);
        }
        else if (call.method.equals(NativeInterface.SEND_PHONE_REG_VCODE)) {
          String phone = call.argument("phone");
          String countryCode = call.argument("country_code");
          String send_result = BLAccountTools.getInstance().sendRegVCode(phone,countryCode);
          result.success(send_result);
        }
        else if (call.method.equals(NativeInterface.SEND_EMAIL_REG_VCODE)) {
          String email = call.argument("email");
          String send_result = BLAccountTools.getInstance().sendRegVCode(email);
          result.success(send_result);
        }
        else if (call.method.equals(NativeInterface.REGIST)) {
          String data = call.argument("data");
          String regist_result = BLAccountTools.getInstance().regist(data);
          result.success(regist_result);
        }
        else if (call.method.equals(NativeInterface.SEND_RESET_PWD_VCODE)) {
          String account = call.argument("account");
          String regist_result = BLAccountTools.getInstance().sendResetPwdVCode(account);
          result.success(regist_result);
        }
        else if (call.method.equals(NativeInterface.RESET_PASSWORD)) {
            String account = call.argument("account");
            String vcode = call.argument("vcode");
            String password = call.argument("password");
            String reset_pwd_result = BLAccountTools.getInstance().resetPwd(account,vcode,password);
            result.success(reset_pwd_result);
        }
        else if (call.method.equals(NativeInterface.SEND_MODIFY_EMAIL_VCODE)) {
            String email = call.argument("email");
            String send_result = BLAccountTools.getInstance().sendModifyEmailVCode(email);
            result.success(send_result);
        }
        else if (call.method.equals(NativeInterface.SEND_MODIFY_PHONE_VCODE)) {
            String phone = call.argument("phone");
            String countryCode = call.argument("country_code");
            String send_result = BLAccountTools.getInstance().sendModifyPhoneVCode(phone,countryCode);
            result.success(send_result);
        }
        else if (call.method.equals(NativeInterface.MODIFY_PHONE)) {
            String phone = call.argument("phone");
            String countryCode = call.argument("country_code");
            String vcode = call.argument("vcode");
            String password = call.argument("password");

            String modify_result = BLAccountTools.getInstance().modifyPhone(phone,countryCode,vcode,password);
            result.success(modify_result);
        }
        else if (call.method.equals(NativeInterface.MODIFY_EMAIL)) {
            String email = call.argument("email");
            String vcode = call.argument("vcode");
            String password = call.argument("password");

            String modify_result = BLAccountTools.getInstance().modifyEmail(email,vcode,password);
            result.success(modify_result);
        }
        else if (call.method.equals(NativeInterface.MODIFY_PASSWORD)) {
            String oldPassword = call.argument("old_password");
            String newPassword = call.argument("new_password");

            String modify_result = BLAccountTools.getInstance().modifyPassword(oldPassword,newPassword);
            result.success(modify_result);
        }
        else if (call.method.equals(NativeInterface.MODIFY_NICKNAME)) {
            String nickname = call.argument("nickname");
            String modify_result = BLAccountTools.getInstance().modifyNickname(nickname);
            result.success(modify_result);
        }
        else if (call.method.equals(NativeInterface.MODIFY_ICON)) {
            String filePath = call.argument("file_path");
            String modify_result = BLAccountTools.getInstance().modifyIcon(filePath);
            result.success(modify_result);
        }
        else if (call.method.equals(NativeInterface.QUERY_USERS_INFO)) {
            String data = call.argument("data");
            String users_info_result = BLAccountTools.getInstance().queryUsersInfo(data);
            result.success(users_info_result);
        }
        else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
